/* Copyright (c) 2000-2003, jMock.org. See bundled LICENSE.txt */
package org.jmock.dynamic;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.*;


/**
 * An object that holds information about a call dispatched to
 * a Mock object for err... mocking.
 * <p/>
 * TODO Simplify this (smgf)
 */
public class Invocation {
    private String name;
    private List parameterTypes;
    private Class returnType;
    private List parameterValues;

    public Invocation(String name, Class[] parameterTypes,
                      Class returnType, Object[] parameterValues) {
        this.name = name;
        this.parameterTypes = Arrays.asList(parameterTypes);
        this.returnType = returnType;
        if (parameterValues == null) {
            this.parameterValues = new ArrayList(0);
        } else {
            this.parameterValues = Arrays.asList(parameterValues);
        }
    }

    public Invocation(Method method, Object[] parameterValues) {
        this(method.getName(), method.getParameterTypes(),
                method.getReturnType(), parameterValues);
    }

    public String getMethodName() {
        return name;
    }

    public List getParameterTypes() {
        return Collections.unmodifiableList(parameterTypes);
    }

    public List getParameterValues() {
        return Collections.unmodifiableList(parameterValues);
    }

    public Class getReturnType() {
        return returnType;
    }

    public String toString() {
        return DynamicUtil.methodToString(name, parameterValues.toArray());
    }

    public boolean equals(Object o) {
        if (o instanceof Invocation) {
            return this.equals((Invocation) o);
        } else {
            return false;
        }
    }

    public int hashCode() {
        return name.hashCode() ^
                listHashCode(parameterTypes) ^
                returnType.hashCode() ^
                listHashCode(parameterValues);
    }

    private int listHashCode(List array) {
        int hashCode = 0;
        for (Iterator i = array.iterator(); i.hasNext();) {
            hashCode ^= i.next().hashCode();
        }
        return hashCode;
    }

    public boolean equals(Invocation call) {
        return call != null
                && name.equals(call.name)
                && parameterTypes.equals(call.parameterTypes)
                && returnType.equals(call.returnType)
                && parameterValues.equals(call.parameterValues);
    }

    boolean isCheckingEqualityOnProxy() {
        return name.equals("equals")
                && parameterValues.size() == 1
                && parameterValues.get(0) != null
                && Proxy.isProxyClass(parameterValues.get(0).getClass());
    }

    boolean isMockNameGetter() {
        return name.equals("getMockName")
                && parameterValues.size() == 0;
    }
}
